"use strict";

exports.__esModule = true;
exports.default = getPrevSiblingNativeNode;

var _host = _interopRequireDefault(require("./host"));

var _toArray = _interopRequireDefault(require("../toArray"));

var _constant = require("../constant");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * This function is usually been used to find the closet previous sibling native node of FragmentComponent.
 * FragmentComponent does not have a native node in the DOM tree, so when it is replaced, the new node has no corresponding location to insert.
 * So we need to look forward from the current mount position of the FragmentComponent to the nearest component which have the native node.
 * @param component
 * @return nativeNode
 */
function getPrevSiblingNativeNode(component) {
  var parent = component;

  while (parent = component.__parentInstance && component.__parentInstance[_constant.INTERNAL]) {
    if (parent instanceof _host.default.Composite) {
      component = parent;
      continue;
    }

    var keys = Object.keys(parent._renderedChildren);

    for (var i = component.__mountIndex - 1; i >= 0; i--) {
      var nativeNode = (0, _toArray.default)(parent._renderedChildren[keys[i]].__getNativeNode());

      if (nativeNode.length > 0) {
        return nativeNode[nativeNode.length - 1];
      }
    }

    if (parent instanceof _host.default.Fragment) {
      component = parent;
    } else {
      return null;
    }
  }
}