"use strict";

exports.__esModule = true;
exports.default = instantiateComponent;
exports.throwInvalidComponentError = throwInvalidComponentError;

var _host = _interopRequireDefault(require("./host"));

var _types = require("../types");

var _error = require("../error");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function instantiateComponent(element) {
  var instance;

  if ((0, _types.isObject)(element) && element !== null && element.type) {
    // Special case string values
    if ((0, _types.isString)(element.type)) {
      instance = new _host.default.Native(element);
    } else {
      instance = new _host.default.Composite(element);
    }
  } else if ((0, _types.isString)(element) || (0, _types.isNumber)(element)) {
    instance = new _host.default.Text(String(element));
  } else if (element === undefined || (0, _types.isNull)(element) || element === false || element === true) {
    instance = new _host.default.Empty();
  } else if ((0, _types.isArray)(element)) {
    instance = new _host.default.Fragment(element);
  } else {
    throwInvalidComponentError(element);
  }

  return instance;
}

function throwInvalidComponentError(element) {
  if (process.env.NODE_ENV === 'production') {
    (0, _error.invokeMinifiedError)(2);
  } else {
    throw new Error("Invalid element type: " + element + ". (current: " + ((0, _types.isObject)(element) && Object.keys(element) || typeof element) + ")");
  }
}