"use strict";

exports.__esModule = true;
exports.default = void 0;

var _host = _interopRequireDefault(require("./host"));

var _ref = require("./ref");

var _instantiateComponent = _interopRequireDefault(require("./instantiateComponent"));

var _shouldUpdateComponent = _interopRequireDefault(require("./shouldUpdateComponent"));

var _getElementKeyName = _interopRequireDefault(require("./getElementKeyName"));

var _getPrevSiblingNativeNode = _interopRequireDefault(require("./getPrevSiblingNativeNode"));

var _instance = _interopRequireDefault(require("./instance"));

var _base = _interopRequireDefault(require("./base"));

var _toArray = _interopRequireDefault(require("../toArray"));

var _types = require("../types");

var _assign = _interopRequireDefault(require("../assign"));

var _constant = require("../constant");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _inheritsLoose(subClass, superClass) { subClass.prototype = Object.create(superClass.prototype); subClass.prototype.constructor = subClass; subClass.__proto__ = superClass; }

var STYLE = 'style';
var CHILDREN = 'children';
var TREE = 'tree';
var EVENT_PREFIX_REGEXP = /^on[A-Z]/;
/**
 * Native Component
 */

var NativeComponent =
/*#__PURE__*/
function (_BaseComponent) {
  _inheritsLoose(NativeComponent, _BaseComponent);

  function NativeComponent() {
    return _BaseComponent.apply(this, arguments) || this;
  }

  var _proto = NativeComponent.prototype;

  _proto.__mountComponent = function __mountComponent(parent, parentInstance, context, nativeNodeMounter) {
    this.__initComponent(parent, parentInstance, context);

    var currentElement = this.__currentElement;
    var props = currentElement.props;
    var type = currentElement.type;
    var children = props[CHILDREN];
    var appendType = props.append || TREE; // Default is tree
    // Clone a copy for style diff

    this.__prevStyleCopy = (0, _assign.default)({}, props[STYLE]);
    var instance = {
      type: type,
      props: props
    };
    instance[_constant.INTERNAL] = this;
    this[_constant.INSTANCE] = instance;

    if (appendType === TREE) {
      // Should after process children when mount by tree mode
      this.__mountChildren(children, context);

      this.__mountNativeNode(nativeNodeMounter);
    } else {
      // Should before process children when mount by node mode
      this.__mountNativeNode(nativeNodeMounter);

      this.__mountChildren(children, context);
    } // Ref acttach


    if (currentElement && currentElement.ref) {
      (0, _ref.attachRef)(currentElement._owner, currentElement.ref, this);
    }

    if (process.env.NODE_ENV !== 'production') {
      _host.default.reconciler.mountComponent(this);
    }

    return instance;
  };

  _proto.__mountChildren = function __mountChildren(children, context) {
    if (children == null) return children;

    var nativeNode = this.__getNativeNode();

    return this.__mountChildrenImpl(nativeNode, (0, _toArray.default)(children), context);
  };

  _proto.__mountChildrenImpl = function __mountChildrenImpl(parent, children, context, nativeNodeMounter) {
    var renderedChildren = this._renderedChildren = {};
    var renderedChildrenImage = [];

    for (var i = 0, l = children.length; i < l; i++) {
      var element = children[i];
      var renderedChild = (0, _instantiateComponent.default)(element);
      var name = (0, _getElementKeyName.default)(renderedChildren, element, i);
      renderedChildren[name] = renderedChild;
      renderedChild.__mountIndex = i; // Mount children

      var mountImage = renderedChild.__mountComponent(parent, this[_constant.INSTANCE], context, nativeNodeMounter);

      renderedChildrenImage.push(mountImage);
    }

    return renderedChildrenImage;
  };

  _proto.__unmountChildren = function __unmountChildren(shouldNotRemoveChild) {
    var renderedChildren = this._renderedChildren;

    if (renderedChildren) {
      for (var name in renderedChildren) {
        var renderedChild = renderedChildren[name];
        renderedChild.unmountComponent(shouldNotRemoveChild);
      }

      this._renderedChildren = null;
    }
  };

  _proto.unmountComponent = function unmountComponent(shouldNotRemoveChild) {
    if (this[_constant.NATIVE_NODE]) {
      var ref = this.__currentElement.ref;

      if (ref) {
        (0, _ref.detachRef)(this.__currentElement._owner, ref, this);
      }

      _instance.default.remove(this[_constant.NATIVE_NODE]);

      if (!shouldNotRemoveChild) {
        _host.default.driver.removeChild(this[_constant.NATIVE_NODE], this._parent); // If the parent node has been removed, child node don't need to be removed


        shouldNotRemoveChild = true;
      }
    }

    this.__unmountChildren(shouldNotRemoveChild);

    this.__prevStyleCopy = null;

    this.__destoryComponent();
  };

  _proto.__updateComponent = function __updateComponent(prevElement, nextElement, prevContext, nextContext) {
    // Replace current element
    this.__currentElement = nextElement;
    (0, _ref.updateRef)(prevElement, nextElement, this);
    var prevProps = prevElement.props;
    var nextProps = nextElement.props;

    this.__updateProperties(prevProps, nextProps); // If the prevElement has no child, mount children directly


    if (prevProps[CHILDREN] == null || (0, _types.isArray)(prevProps[CHILDREN]) && prevProps[CHILDREN].length === 0) {
      this.__mountChildren(nextProps[CHILDREN], nextContext);
    } else {
      this.__updateChildren(nextProps[CHILDREN], nextContext);
    }

    if (process.env.NODE_ENV !== 'production') {
      _host.default.reconciler.receiveComponent(this);
    }
  };

  _proto.__updateProperties = function __updateProperties(prevProps, nextProps) {
    var propKey;
    var styleName;
    var styleUpdates;
    var driver = _host.default.driver;

    var nativeNode = this.__getNativeNode();

    for (propKey in prevProps) {
      // Continue children and null value prop or nextProps has some propKey that do noting
      if (propKey === CHILDREN || prevProps[propKey] == null || // Use hasOwnProperty here for avoid propKey name is some with method name in object proptotype
      nextProps.hasOwnProperty(propKey)) {
        continue;
      }

      if (propKey === STYLE) {
        // Remove all style
        var lastStyle = this.__prevStyleCopy;

        for (styleName in lastStyle) {
          styleUpdates = styleUpdates || {};
          styleUpdates[styleName] = '';
        }

        this.__prevStyleCopy = null;
      } else if (EVENT_PREFIX_REGEXP.test(propKey)) {
        // Remove event
        var eventListener = prevProps[propKey];

        if ((0, _types.isFunction)(eventListener)) {
          driver.removeEventListener(nativeNode, propKey.slice(2).toLowerCase(), eventListener);
        }
      } else {
        // Remove attribute
        driver.removeAttribute(nativeNode, propKey, prevProps[propKey]);
      }
    }

    for (propKey in nextProps) {
      var nextProp = nextProps[propKey];
      var prevProp = propKey === STYLE ? this.__prevStyleCopy : prevProps != null ? prevProps[propKey] : undefined; // Continue children or prevProp equal nextProp

      if (propKey === CHILDREN || prevProp === nextProp || nextProp == null && prevProp == null) {
        continue;
      } // Update style


      if (propKey === STYLE) {
        if (nextProp) {
          // Clone property
          nextProp = this.__prevStyleCopy = (0, _assign.default)({}, nextProp);
        } else {
          this.__prevStyleCopy = null;
        }

        if (prevProp != null) {
          // Unset styles on `prevProp` but not on `nextProp`.
          for (styleName in prevProp) {
            if (!nextProp || !nextProp[styleName] && nextProp[styleName] !== 0) {
              styleUpdates = styleUpdates || {};
              styleUpdates[styleName] = '';
            }
          } // Update styles that changed since `prevProp`.


          for (styleName in nextProp) {
            if (prevProp[styleName] !== nextProp[styleName]) {
              styleUpdates = styleUpdates || {};
              styleUpdates[styleName] = nextProp[styleName];
            }
          }
        } else {
          // Assign next prop when prev style is null
          styleUpdates = nextProp;
        }
      } else if (EVENT_PREFIX_REGEXP.test(propKey)) {
        // Update event binding
        var eventName = propKey.slice(2).toLowerCase();

        if ((0, _types.isFunction)(prevProp)) {
          driver.removeEventListener(nativeNode, eventName, prevProp, nextProps);
        }

        if ((0, _types.isFunction)(nextProp)) {
          driver.addEventListener(nativeNode, eventName, nextProp, nextProps);
        }
      } else {
        // Update other property
        if (nextProp != null) {
          driver.setAttribute(nativeNode, propKey, nextProp);
        } else {
          driver.removeAttribute(nativeNode, propKey, prevProps[propKey]);
        }

        if (process.env.NODE_ENV !== 'production') {
          var _payload;

          _host.default.measurer && _host.default.measurer.recordOperation({
            instanceID: this._mountID,
            type: 'update attribute',
            payload: (_payload = {}, _payload[propKey] = nextProp, _payload)
          });
        }
      }
    }

    if (styleUpdates) {
      if (process.env.NODE_ENV !== 'production') {
        _host.default.measurer && _host.default.measurer.recordOperation({
          instanceID: this._mountID,
          type: 'update style',
          payload: styleUpdates
        });
      }

      driver.setStyle(nativeNode, styleUpdates);
    }
  };

  _proto.__updateChildren = function __updateChildren(nextChildrenElements, context) {
    var _this = this;

    // prev rendered children
    var prevChildren = this._renderedChildren;
    var driver = _host.default.driver;

    if (nextChildrenElements == null && prevChildren == null) {
      return;
    }

    var nextChildren = {};

    if (nextChildrenElements != null) {
      nextChildrenElements = (0, _toArray.default)(nextChildrenElements); // Update next children elements

      for (var index = 0, length = nextChildrenElements.length; index < length; index++) {
        var nextElement = nextChildrenElements[index];
        var name = (0, _getElementKeyName.default)(nextChildren, nextElement, index);
        var prevChild = prevChildren && prevChildren[name];
        var prevElement = prevChild && prevChild.__currentElement;
        var prevContext = prevChild && prevChild._context; // Try to update between the two of some name that has some element type,
        // and move child in next children loop if need

        if (prevChild != null && (0, _shouldUpdateComponent.default)(prevElement, nextElement)) {
          if (prevElement !== nextElement || prevContext !== context) {
            // Pass the same context when updating children
            prevChild.__updateComponent(prevElement, nextElement, context, context);
          }

          nextChildren[name] = prevChild;
        } else {
          // Unmount the prevChild when some name with nextChild but different element type,
          // and move child node in next children loop
          if (prevChild) {
            prevChild.__unmount = true;
          } // The child must be instantiated before it's mounted.


          nextChildren[name] = (0, _instantiateComponent.default)(nextElement);
        }
      }
    }

    var prevFirstChild;
    var prevFirstNativeNode;
    var shouldUnmountPrevFirstChild; // Directly remove all children from component, if nextChildren is empty (null, [], '').
    // `driver.removeChildren` is optional driver protocol.

    var shouldRemoveAllChildren = Boolean(driver.removeChildren // nextChildElements == null or nextChildElements is empty
    && ((0, _types.isNull)(nextChildrenElements) || nextChildrenElements && !nextChildrenElements.length)); // Unmount children that are no longer present.

    if (prevChildren != null) {
      for (var _name in prevChildren) {
        var _prevChild = prevChildren[_name];
        var shouldUnmount = _prevChild.__unmount || !nextChildren[_name]; // Store old first child ref for append node ahead and maybe delay remove it

        if (!prevFirstChild) {
          shouldUnmountPrevFirstChild = shouldUnmount;
          prevFirstChild = _prevChild;
          prevFirstNativeNode = prevFirstChild.__getNativeNode();

          if ((0, _types.isArray)(prevFirstNativeNode)) {
            prevFirstNativeNode = prevFirstNativeNode[0];
          }
        } else if (shouldUnmount) {
          _prevChild.unmountComponent(shouldRemoveAllChildren);
        }
      }
    }

    if (nextChildren != null) {
      // `nextIndex` will increment for each child in `nextChildren`
      var nextIndex = 0;
      var lastPlacedNode = null;
      var nextNativeNodes = [];
      var isFragmentAsParent = false;

      var insertNodes = function insertNodes(nativeNodes, parent) {
        // The nativeNodes maybe fragment, so convert to array type
        nativeNodes = (0, _toArray.default)(nativeNodes);
        var prevSiblingNativeNode; // Only parent is fragment need to get the prev sibling node

        if (isFragmentAsParent) {
          prevSiblingNativeNode = (0, _getPrevSiblingNativeNode.default)(_this);
        }

        for (var i = 0, l = nativeNodes.length; i < l; i++) {
          if (lastPlacedNode) {
            // Should reverse order when insert new child after lastPlacedNode:
            // [lastPlacedNode, *newChild1, *newChild2]
            driver.insertAfter(nativeNodes[l - i - 1], lastPlacedNode);
          } else if (prevFirstNativeNode) {
            // [*newChild1, *newChild2, prevFirstNativeNode]
            driver.insertBefore(nativeNodes[i], prevFirstNativeNode);
          } else if (prevSiblingNativeNode) {
            // If parent is fragment, find nativeNode previous sibling node
            driver.insertAfter(nativeNodes[i], prevSiblingNativeNode);
          } else if (parent) {
            // [*newChild1, *newChild2]
            driver.appendChild(nativeNodes[i], parent);
          }
        }
      };

      for (var _name2 in nextChildren) {
        var nextChild = nextChildren[_name2];

        var _prevChild2 = prevChildren && prevChildren[_name2]; // Try to move the some key prevChild but current not at the some position


        if (_prevChild2 === nextChild) {
          var prevChildNativeNode = _prevChild2.__getNativeNode();

          if (_prevChild2.__mountIndex !== nextIndex) {
            insertNodes(prevChildNativeNode);
          }
        } else {
          // Mount nextChild that in prevChildren there has no some name
          var parent = this.__getNativeNode(); // Fragment extended native component, so if parent is fragment should get this._parent


          if ((0, _types.isArray)(parent)) {
            isFragmentAsParent = true;
            parent = this._parent;
          }

          nextChild.__mountComponent(parent, this[_constant.INSTANCE], context, insertNodes // Insert nodes mounter
          );
        } // Update to the latest mount order


        nextChild.__mountIndex = nextIndex++; // Get the last child

        lastPlacedNode = nextChild.__getNativeNode(); // Push to nextNativeNodes

        if ((0, _types.isArray)(lastPlacedNode)) {
          nextNativeNodes = nextNativeNodes.concat(lastPlacedNode);
          lastPlacedNode = lastPlacedNode[lastPlacedNode.length - 1];
        } else {
          nextNativeNodes.push(lastPlacedNode);
        }
      } // Sync update native refs


      if ((0, _types.isArray)(this[_constant.NATIVE_NODE])) {
        // Clear all and push the new array
        this[_constant.NATIVE_NODE].length = 0;
        (0, _assign.default)(this[_constant.NATIVE_NODE], nextNativeNodes);
      }
    }

    if (shouldUnmountPrevFirstChild) {
      prevFirstChild.unmountComponent(shouldRemoveAllChildren);
    }

    if (shouldRemoveAllChildren) {
      driver.removeChildren(this[_constant.NATIVE_NODE]);
    }

    this._renderedChildren = nextChildren;
  };

  _proto.__createNativeNode = function __createNativeNode() {
    var instance = this[_constant.INSTANCE];

    var nativeNode = _host.default.driver.createElement(instance.type, instance.props, this);

    _instance.default.set(nativeNode, instance);

    return nativeNode;
  };

  return NativeComponent;
}(_base.default);

exports.default = NativeComponent;