"use strict";

exports.__esModule = true;
exports.default = void 0;

var _host = _interopRequireDefault(require("./host"));

var _component = _interopRequireDefault(require("./component"));

var _invokeFunctionsWithContext = _interopRequireDefault(require("../invokeFunctionsWithContext"));

var _error = require("../error");

var _constant = require("../constant");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _inheritsLoose(subClass, superClass) { subClass.prototype = Object.create(superClass.prototype); subClass.prototype.constructor = subClass; subClass.__proto__ = superClass; }

var RE_RENDER_LIMIT = 24;
/**
 * Functional Reactive Component Class Wrapper
 */

var ReactiveComponent =
/*#__PURE__*/
function (_Component) {
  _inheritsLoose(ReactiveComponent, _Component);

  function ReactiveComponent(pureRender, ref) {
    var _this;

    _this = _Component.call(this) || this; // Marked ReactiveComponent.

    _this.__isReactiveComponent = true; // A pure function

    _this.__render = pureRender;
    _this._hookID = 0; // Number of rerenders

    _this.__reRenders = 0;
    _this._hooks = {}; // Handles store

    _this.didMount = [];
    _this.didUpdate = [];
    _this.willUnmount = []; // Is render scheduled

    _this.__isScheduled = false;
    _this.__shouldUpdate = false;
    _this.__children = null;
    _this.__dependencies = {};
    _this.state = {};

    if (pureRender.forwardRef) {
      _this.prevForwardRef = _this.forwardRef = ref;
    }

    var compares = pureRender.compares;

    if (compares) {
      _this.shouldComponentUpdate = function (nextProps) {
        // Process composed compare
        var arePropsEqual = true; // Compare push in and pop out

        for (var i = compares.length - 1; i > -1; i--) {
          if (arePropsEqual = compares[i](_this.props, nextProps)) {
            break;
          }
        }

        return !arePropsEqual || _this.prevForwardRef !== _this.forwardRef;
      };
    }

    return _this;
  }

  var _proto = ReactiveComponent.prototype;

  _proto.getHooks = function getHooks() {
    return this._hooks;
  };

  _proto.getHookID = function getHookID() {
    return ++this._hookID;
  };

  _proto.readContext = function readContext(context) {
    var _this2 = this;

    var Provider = context.Provider;
    var contextProp = Provider.contextProp;
    var contextItem = this.__dependencies[contextProp];

    if (!contextItem) {
      var readEmitter = Provider.readEmitter;
      var contextEmitter = readEmitter(this);
      contextItem = {
        emitter: contextEmitter,
        renderedContext: contextEmitter.value
      };

      var contextUpdater = function contextUpdater(newContext) {
        if (newContext !== contextItem.renderedContext) {
          _this2.__shouldUpdate = true;

          _this2.update();
        }
      };

      contextEmitter.on(contextUpdater);
      this.willUnmount.push(contextEmitter.off.bind(contextEmitter, contextUpdater));
      this.__dependencies[contextProp] = contextItem;
    }

    return contextItem.renderedContext = contextItem.emitter.value;
  };

  _proto.componentWillMount = function componentWillMount() {
    this.__shouldUpdate = true;
  };

  _proto.componentDidMount = function componentDidMount() {
    (0, _invokeFunctionsWithContext.default)(this.didMount);
  };

  _proto.componentWillReceiveProps = function componentWillReceiveProps() {
    this.__shouldUpdate = true;
  };

  _proto.componentDidUpdate = function componentDidUpdate() {
    (0, _invokeFunctionsWithContext.default)(this.didUpdate);
  };

  _proto.componentWillUnmount = function componentWillUnmount() {
    (0, _invokeFunctionsWithContext.default)(this.willUnmount);
  };

  _proto.update = function update() {
    this[_constant.INTERNAL].__isPendingForceUpdate = true;
    this.setState({});
  };

  _proto.render = function render() {
    if (process.env.NODE_ENV !== 'production') {
      _host.default.measurer && _host.default.measurer.beforeRender();
    }

    this._hookID = 0;
    this.__reRenders = 0;
    this.__isScheduled = false;

    var children = this.__render(this.props, this.forwardRef ? this.forwardRef : this.context);

    while (this.__isScheduled) {
      this.__reRenders++;

      if (this.__reRenders > RE_RENDER_LIMIT) {
        if (process.env.NODE_ENV !== 'production') {
          throw new Error('Too many re-renders, the number of renders is limited to prevent an infinite loop.');
        } else {
          (0, _error.invokeMinifiedError)(4);
        }
      }

      this._hookID = 0;
      this.__isScheduled = false;
      children = this.__render(this.props, this.forwardRef ? this.forwardRef : this.context);
    }

    if (this.__shouldUpdate) {
      this.__children = children;
      this.__shouldUpdate = false;
    }

    return this.__children;
  };

  return ReactiveComponent;
}(_component.default);

exports.default = ReactiveComponent;